from django.conf import settings
from django.db.models.loading import get_model
from django.shortcuts import get_object_or_404
from django.views import generic

from stardate.models import Blog
from stardate.utils import get_post_model

Post = get_post_model()


class PostViewMixin(object):
    model = Post
    date_field = 'publish'

    def get_queryset(self):
        blog = get_object_or_404(Blog, slug__iexact=self.kwargs['blog_slug'])
        return Post.objects.published().filter(blog=blog)


class PostArchiveIndex(PostViewMixin, generic.ArchiveIndexView):
    context_object_name = 'post_list'

    def get_context_data(self, **kwargs):
        context = super(PostArchiveIndex, self).get_context_data(**kwargs)
        context['blog'] = Blog.objects.get(slug__iexact=self.kwargs['blog_slug'])
        return context


class PostYearArchive(PostViewMixin, generic.YearArchiveView):
    make_object_list = True


class PostMonthArchive(PostViewMixin, generic.MonthArchiveView):
    pass


class PostDayArchive(PostViewMixin, generic.DayArchiveView):
    pass


class PostDetail(PostViewMixin, generic.DateDetailView):
    context_object_name = 'post'
    slug_url_kwarg = 'post_slug'


class PostEdit(PostViewMixin, generic.UpdateView):
    context_object_name = 'post'
    slug_url_kwarg = 'post_slug'



from django.contrib import admin

from stardate.utils import get_post_model
from stardate.models import Blog
from stardate.forms import BlogForm, PostForm

Post = get_post_model()


class BlogAdmin(admin.ModelAdmin):
    exclude = ['user', ]
    fields = ('social_auth', 'backend_file', 'name', 'slug', 'authors')
    form = BlogForm
    prepopulated_fields = {'slug': ('name',)}

    def save_model(self, request, obj, form, change):
        """
        When creating a new object, set the user field.
        """
        if not change:
            obj.user = request.user
        obj.save()


class PostAdmin(admin.ModelAdmin):
    date_hierarchy = 'publish'
    fields = ('blog', 'title', 'slug', 'body', 'publish', 'authors')
    form = PostForm
    list_display = ('title', 'publish', 'blog', 'deleted')
    list_filter = ('blog', 'publish',)
    prepopulated_fields = {'slug': ('title',)}


admin.site.register(Blog, BlogAdmin)
admin.site.register(Post, PostAdmin)



import logging

from django.core.management.base import BaseCommand

from social.apps.django_app.default.models import UserSocialAuth

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    """
    Updates the extra_data key on a UserSocialAuth object created with
    django-social-auth to a compatible UserSocialAuth object for
    python-social-auth.
    """

    def handle(self, *args, **options):
        for social_auth in UserSocialAuth.objects.all():
            self.convert_social_auth(social_auth)

    def convert_social_auth(self, auth_obj):
        access_token = auth_obj.extra_data['access_token']

        auth_obj.extra_data['access_token'] = {
            'oauth_token_secret': access_token.split('&')[0].split('=')[1],
            'oauth_token': access_token.split('&')[1].split('=')[1],
            'uid': auth_obj.uid,
        }
        auth_obj.extra_data['expires'] = 'null'
        auth_obj.extra_data['id'] = 'null'

        auth_obj.save()



from django.test import TestCase
from django.test.client import Client

# from stardate.feeds import LatestPostsFeed
from stardate.tests.factories import create_blog, create_post
from stardate.utils import get_post_model


Post = get_post_model()


class LatestFeedTestCase(TestCase):
    def setUp(self):
        b = create_blog()
        create_post(
            blog=b,
            body='# Headline\n\nAnd some text.\n'
        )
        self.client = Client()

    def test_feed(self):
        response = self.client.get('/test-blog/rss/')

        self.assertEqual(response.status_code, 200)



from django.conf.urls import patterns, url

from stardate.feeds import LatestPostsFeed
from stardate import views


urlpatterns = patterns('',
    url(r'^$', views.PostArchiveIndex.as_view(), name='post-archive-index'),
    url(r'rss/$', LatestPostsFeed(), name='post-feed'),
    url(r'^(?P<year>\d{4})/$', views.PostYearArchive.as_view(), name='post-archive-year'),
    url(r'^(?P<year>\d{4})/(?P<month>\w{3})/$', views.PostMonthArchive.as_view(), name='post-archive-month'),
    url(r'^(?P<year>\d{4})/(?P<month>\w{3})/(?P<day>\d{1,2})/$', views.PostDayArchive.as_view(), name='post-archive-day'),
    url(r'^(?P<year>\d{4})/(?P<month>\w{3})/(?P<day>\d{1,2})/(?P<post_slug>[-\w]+)/$',
        views.PostDetail.as_view(), name='post-detail'),
    url(r'^(?P<post_slug>[-\w]+)/edit/$', views.PostEdit.as_view(), name='post-edit')
)



from __future__ import absolute_import
import os



from stardate.backends import StardateBackend
from stardate.parsers import FileParser


class LocalFileBackend(StardateBackend):
    def __init__(self):
        self.name = u'localfile'
        self.parser = FileParser()
        self.social_auth = None

    def write_file(self, file_path, content):
        with open(file_path, 'w') as f:
            f.write(content)

    def get_file(self, path):
        if os.path.exists(path):
            with open(path, 'r') as f:
                content = f.read()
        else:
            content = None
        return content

    def get_post(self, path):
        if os.path.exists(path):
            content = self.get_file(path)
            post = self.parser.parser(post)
        else:
            post = {}
        return post

    def get_posts(self, path):
        """
        Fetch post dictionaries from single file or directory
        """
        # First try to parse it as a directory
        # If we fail, parse it as a file
        if os.path.isfile(path):
            content = self.get_file(path)
            posts = self.parser.unpack(content)
        elif os.path.isdir(path):
            posts = []
            file_list = self._list_path(path)
            for filename in file_list:
                file_path = os.path.join(path, filename)
                content = self.get_file(file_path)
                post = self.parser.parse(content)
                posts.append(post)
        else:
            raise Exception('File does not exist')
        return posts

    def _list_path(self, path):
        return os.listdir(path)

    def set_social_auth(self, *args, **kwargs):
        return

    def _get_post_path(self, folder, post):
        """
        Dynamically guess post file path from slug / blog folder
        """
        filename = post['slug']
        filename = '{0}.md'.format(filename)
        path = os.path.join(folder, filename)
        return path



import datetime

from django.core.exceptions import ValidationError
from django.test import TestCase
from django.utils import timezone

from stardate.models import Blog
from stardate.tests.factories import create_blog, create_post
from stardate.utils import get_post_model

Post = get_post_model()


class BlogTestCase(TestCase):
    def setUp(self):
        self.blog = create_blog(
            name="My test blog",
            backend_class="stardate.tests.mock_backends.MockDropboxBackend",
        )

        pub_date_1 = datetime.datetime(2012, 1, 2, 8, 0, tzinfo=timezone.utc)
        pub_date_2 = datetime.datetime(2012, 1, 3, 8, 0, tzinfo=timezone.utc)
        create_post(blog=self.blog, publish=pub_date_1)
        create_post(blog=self.blog, title="Test post 2 title", publish=pub_date_2)

    def tearDown(self):
        Blog.objects.all().delete()

    def test_blog_has_slug(self):
        self.assertTrue(self.blog.slug)
        self.assertEqual(self.blog.slug, 'test-blog')

    def test_get_post_model(self):
        from core.models import Post as PostClass
        self.assertIsInstance(Post(), PostClass)

        with self.settings(STARDATE_POST_MODEL='stardate.CustomPost'):
            from core.models import CustomPost
            p = get_post_model()
            self.assertIsInstance(p(), CustomPost)

    def test_get_serialized_posts(self):
        posts = self.blog.get_serialized_posts()
        self.assertEqual(len(posts), 2)

    def test_get_posts(self):
        post_list = self.blog.get_posts()
        self.assertTrue(len(post_list), 2)

    def test_get_next_post(self):
        first_post = Post.objects.get(title="Test post title")
        self.assertEqual(first_post.title, "Test post title")
        self.assertTrue(first_post.get_next_post())

        last_post = Post.objects.get(title="Test post 2 title")
        self.assertEqual(last_post.title, "Test post 2 title")
        self.assertFalse(last_post.get_next_post())

        self.assertEqual(first_post.get_next_post(), last_post)

    def test_get_prev_post(self):
        first_post = Post.objects.get(title="Test post title")
        last_post = Post.objects.get(title="Test post 2 title")

        self.assertEqual(last_post.get_prev_post(), first_post)
        self.assertFalse(first_post.get_prev_post())

    def test_save_invalid_post(self):
        data = {
            'blog': self.blog
        }
        p = Post(**data)
        self.assertRaises(ValidationError, p.save)

    def test_post_marked_deleted_is_removed(self):
        p = self.blog.get_posts().get(title="Test post title")
        p.mark_deleted()
        p.save()  # Probably bad
        self.assertTrue(p.deleted)
        self.assertTrue(self.blog.get_posts().get(title="Test post title").deleted)
        self.assertTrue(len(self.blog.get_serialized_posts()), 1)

    def test_removed_post_is_deleted(self):
        post_list = self.blog.get_serialized_posts()
        post_list.remove(post_list[0])
        self.assertTrue(len(post_list), 1)



import datetime
import uuid

from django.contrib.auth.models import User
from django.test import TestCase
from django.utils import timezone

from social.apps.django_app.default.models import UserSocialAuth

from stardate.models import Blog
from stardate.parsers import FileParser
from stardate.tests.factories import create_blog
from stardate.tests.mock_backends import MockDropboxClient


class FileParserTestCase(TestCase):
    def setUp(self):
        self.parser = FileParser()
        self.test_string = "publish: 2012-01-02 12:00 AM\ntitle: Tingling of the spine\n\n\nExtraordinary claims require extraordinary evidence!"

    def tearDown(self):
        Blog.objects.all().delete()
        User.objects.all().delete()
        UserSocialAuth.objects.all().delete()

    def test_pack(self):
        blog = create_blog()
        blog.backend.client_class = MockDropboxClient
        post_list = [
            {
                'title': 'My first post',
                'created': datetime.datetime.now(),
                'stardate': uuid.uuid1(),
                'body': 'This is the first post.'
            },
            {
                'title': 'My second post',
                'created': datetime.datetime.now(),
                'stardate': uuid.uuid1(),
                'body': 'This is the second post.'
            },
        ]
        packed = self.parser.pack(post_list)

        self.assertIsInstance(post_list, list)
        self.assertEqual(len(post_list), 2)
        self.assertIsInstance(packed, basestring)

        self.assertTrue(u'title: {0}'.format(post_list[0]['title']) in packed)
        self.assertTrue(u'title: {0}'.format(post_list[1]['title']) in packed)
        self.assertTrue(u'created: {0}'.format(post_list[0]['created']) in packed)
        self.assertTrue(u'created: {0}'.format(post_list[1]['created']) in packed)
        self.assertTrue(u'stardate: {0}'.format(post_list[0]['stardate']) in packed)
        self.assertTrue(u'stardate: {0}'.format(post_list[1]['stardate']) in packed)
        self.assertTrue(u'\n\n\n{0}'.format(post_list[0]['body']) in packed)
        self.assertTrue(u'\n\n\n{0}'.format(post_list[1]['body']) in packed)

    def test_parse(self):
        parsed = self.parser.parse(self.test_string)

        self.assertEqual(parsed['title'], 'Tingling of the spine')
        self.assertEqual(parsed['publish'], datetime.datetime(2012, 1, 2, 8, 0, tzinfo=timezone.utc))
        self.assertEqual(parsed['body'], 'Extraordinary claims require extraordinary evidence!')

        # Check that extra_field is parsed
        string = u"title: My title\nextra_field: Something arbitrary\n\n\nThe body.\n"
        parsed = self.parser.parse(string)
        self.assertTrue(parsed.has_key('title'))
        self.assertTrue(parsed.has_key('extra_field'))

    def test_render(self):
        test_stardate = uuid.uuid1()
        dict_to_render = {
            'body': 'The body.',
            'publish': datetime.datetime(2013, 6, 1, 0, 0),
            'stardate': test_stardate,
            'title': 'Test title',
        }
        dict_without_publish = dict_to_render.copy()
        dict_without_publish.pop('publish', None)

        packed = self.parser.pack([dict_to_render])
        rendered = self.parser.render(dict_to_render)
        self.assertEqual(packed, rendered)

        packed = self.parser.pack([dict_without_publish])
        rendered = self.parser.render(dict_without_publish)
        self.assertEqual(packed, rendered)

        dict_to_render['extra_field'] = u'Something arbitrary'
        rendered = self.parser.render(dict_to_render)
        packed = self.parser.pack([dict_to_render])
        self.assertEqual(rendered, packed)
        self.assertTrue('extra_field: Something arbitrary\n' in rendered)

    def test_unpack(self):
        content = self.test_string
        post_list = self.parser.unpack(content)

        #The file has one post to unpack
        self.assertEqual(len(post_list), 1)
        self.assertEqual(post_list[0].get('title'), 'Tingling of the spine')
        self.assertEqual(post_list[0].get('publish'), datetime.datetime(2012, 1, 2, 8, 0, tzinfo=timezone.utc))
        self.assertEqual(post_list[0].get('body'), 'Extraordinary claims require extraordinary evidence!')



import json
import mimetypes

from stardate.backends.dropbox import DropboxBackend
from stardate.backends.local_file import LocalFileBackend


class MockDropboxClient(object):
    entry_data = [['/test_file.md', {
        'size': '452 bytes',
        'rev': '7cb07486f6b',
        'thumb_exists': False,
        'bytes': 452,
        'modified': 'Mon, 16 Jul 2012 00:32:36 +0000',
        'mime_type': 'application/octet-stream',
        'path': '/test_file.md',
        'is_dir': False,
        'icon': 'page_white',
        'root': 'app_folder',
        'client_mtime': 'Mon, 16 Jul 2012 00:32:36 +0000',
        'revision': 1995}]]

    data = json.dumps({
        'entries': entry_data,
        'reset': False,
        'cursor': 'VAU6GZG5NK31AW2YD8H7UDWE0W74VV',
        'has_more': False
    })

    metadata_data = json.dumps({
        'bytes': 0,
        'contents': [
            {
                'bytes': 0,
                'icon': 'folder',
                'is_dir': True,
                'modified': 'Thu, 25 Aug 2011 00:03:15 +0000',
                'path': '/Sample Folder',
                'rev': '803beb471',
                'revision': 8,
                'root': 'dropbox',
                'size': '0 bytes',
                'thumb_exists': False
            },
            {
                'bytes': 77,
                'icon': 'page_white_text',
                'is_dir': False,
                'mime_type': 'text/plain',
                'modified': 'Wed, 20 Jul 2011 22:04:50 +0000',
                'path': '/magnum-opus.txt',
                'rev': '362e2029684fe',
                'revision': 221922,
                'root': 'dropbox',
                'size': '77 bytes',
                'thumb_exists': False
            }
        ],
        'hash': 'efdac89c4da886a9cece1927e6c22977',
        'icon': 'folder',
        'is_dir': True,
        'path': '/',
        'root': 'app_folder',
        'size': '0 bytes',
        'thumb_exists': False
    })

    metadata_data_2 = json.dumps({
        'bytes': 0,
        'contents': [
            {
                'bytes': 77,
                'icon': 'page_white_text',
                'is_dir': False,
                'mime_type': 'text/plain',
                'modified': 'Wed, 20 Jul 2011 22:04:50 +0000',
                'path': '/Sample Folder/magnum-opus.txt',
                'rev': '362e2029684fe',
                'revision': 221922,
                'root': 'dropbox',
                'size': '77 bytes',
                'thumb_exists': False
            }
        ],
        'hash': 'efdac89c4da886a9cece1927e6c22977',
        'icon': 'folder',
        'is_dir': True,
        'path': '/Sample Folder',
        'root': 'app_folder',
        'size': '0 bytes',
        'thumb_exists': False
    })

    def __init__(self, sess=None):
        self.sess = sess

    def delta(self, cursor):
        return json.loads(self.data)

    def get_file(self, path):
        f = open(path, 'r')
        return f

    def metadata(self, path='/', hash=None):
        data = {}
        if path == '/Sample Folder':
            data = json.loads(self.metadata_data_2)
        if path == '/':
            data = json.loads(self.metadata_data)
        return data

    def put_file(self, full_path, file_obj, overwrite=False, parent_rev=None):
        f = open(full_path, 'w')
        f.write(file_obj)
        f.close()
        resp = {
            'path': full_path,
            'mime_type': mimetypes.guess_type(full_path),
        }
        return resp


class MockDropboxBackend(DropboxBackend):
    def __init__(self, client_class=MockDropboxClient):
        return super(MockDropboxBackend, self).__init__(client_class=client_class)


class MockLocalFileBackend(LocalFileBackend):
    pass



from __future__ import absolute_import
import logging
import os

from django.conf import settings
from django.core.cache import cache
from django.core.exceptions import ImproperlyConfigured

from dropbox import client, rest, session

from stardate.backends import StardateBackend
from stardate.parsers import FileParser

def get_settings_or_env(var_name):
    """
    Get a setting from the settings file or an env variable
    """
    value = getattr(settings, var_name, None)
    if not value:
        try:
            return os.environ[var_name]
        except KeyError:
            error_msg = "Could not find setting or ENV variable {0}".format(var_name)
            raise ImproperlyConfigured(error_msg)
    return value

APP_KEY = get_settings_or_env('DROPBOX_APP_KEY')
APP_SECRET = get_settings_or_env('DROPBOX_APP_SECRET')
ACCESS_TYPE = get_settings_or_env('DROPBOX_ACCESS_TYPE')

logger = logging.getLogger('stardate')


class DropboxBackend(StardateBackend):
    def __init__(self, client_class=client.DropboxClient):
        self.client = None
        self.client_class = client_class
        self.cursor = self.get_cursor()
        self.name = u'dropbox'
        self.parser = FileParser()
        self.social_auth = None

    def get_file(self, path):
        return self.client.get_file(path).read()

    def write_file(self, file_path, content):
        return self.client.put_file(file_path, content, overwrite=True)

    def get_post(self, path):
        try:
            content = self.get_file(path)
            post = self.parser.parser(post)
        except Exception:
            post = {}
        return post

    def get_access_token(self):
        return self.social_auth.extra_data['access_token']

    def get_cursor(self):
        try:
            cursor = self.social_auth.extra_data['cursor']
        except (AttributeError, KeyError):
            cursor = None
        return cursor

    def get_dropbox_client(self):
        sess = session.DropboxSession(APP_KEY, APP_SECRET, ACCESS_TYPE)
        token = self.get_access_token()
        sess.set_token(token['oauth_token'], token['oauth_token_secret'])
        return self.client_class(sess)

    def delta(self):
        delta = self.client.delta(cursor=self.cursor)
        self.save_cursor(delta.get('cursor'))
        return delta

    def _list_path(self, path='/', hash=None):
        """
        List the contents of a path on the backend. Each path can be passed
        a `hash` argument that determines if anything new for the specified
        path is returned.

        """
        paths = cache.get('paths', [])
        meta_hash = cache.get('hash', None)

        try:
            meta = self.client.metadata(path, hash=meta_hash)
            cache.delete('paths')
            cache.set('hash', meta['hash'])
        except rest.ErrorResponse, e:
            if e.status == 304:
                return paths
            raise e

        for content in meta.get('contents', []):
            paths.append(content['path'])
        cache.set('paths', paths)
        return paths

    def get_source_list(self):
        paths = cache.get('paths') or self._list_path()
        source_list = ((0, u'---'),)

        #  Instead of using the index, could use slugify
        try:
            for index, path in enumerate(paths):
                source_list += ((index + 1), path),
        except (AttributeError, TypeError):
            pass
        return source_list

    def save_cursor(self, cursor):
        self.social_auth.extra_data['cursor'] = cursor
        self.social_auth.save()
        self.cursor = self.get_cursor()

    def set_social_auth(self, social_auth):
        self.social_auth = social_auth
        self.client = self.get_dropbox_client()

    def get_post_path(self, folder, post):
        """
        Dynamically guess post file path from slug / blog folder
        """
        filename = post.slug
        filename = '{0}.md'.format(filename)
        path = os.path.join(folder, filename)
        return path

    def push_blog_file(self, blog_path, posts):
        """
        Update posts in a single blog file
        """
        try:
            content = self.client.get_file(blog_path).read()
            remote_posts = self.parser.unpack(content)
        except Exception:
            remote_posts = []

        # Use serialized version of posts to find
        # and update
        local_posts = self.serialize_posts(posts)

        # Update remote_posts with local versions
        ## FIXME: n^2 crawl, use stardate as keys
        ## in dicts instead of lists?
        for local_post in local_posts:
            exists = False
            # First we try to see if match by stardate exists
            for remote_post in remote_posts:
                if 'stardate' in remote_post:
                    if local_post['stardate'] == remote_post['stardate']:
                        exists = True
                        remote_post.update(local_post)
                        break

            # If post was created remotely and was pulled in
            # then it has a stardate, but remote post does not.
            # Try to match up using title
            if not exists:
                for remote_post in remote_posts:
                    if 'title' in remote_post:
                        if local_post['title'] == remote_post['title']:
                            exists = True
                            remote_post.update(local_post)
                            break

            # Add new remote post if it does not exist yet
            if not exists:
                remote_posts.append(local_post)

        # Turn post list back into string
        content = self.parser.pack(remote_posts)
        return self.client.put_file(blog_path, content, overwrite=True)

    def push_post_files(self, blog_dir, posts):
        """
        Update posts in multiple files
        """
        responses = []
        for post in posts:
            # Generate the post file path dynamically
            post_path = self.get_post_path(blog_dir, post)

            # Get the existing remote post as a post dict
            try:
                remote_post = self.client.get_file(post_path).read()
            except Exception:
                remote_post = {}
            remote_post = self.parser.parse(remote_post)

            # Turn local post into post dict
            local_post = self.serialize_posts([post])[0]


            # Update the contents of the remote post
            remote_post.update(local_post)
            content = self.parser.render(remote_post)
            response = self.client.put_file(post_path, content, overwrite=True)

            # Log the result
            responses.append(response)
        return responses

    def push(self, posts):
        """
        Sync one or more posts with remote Dropbox

        posts: List of Post object instances
        """
        # Grab the file or folder path associated
        # with a blog
        blog_path = posts[0].blog.backend_file

        # Separate blog path into directory and filename
        blog_dir, blog_file = os.path.split(blog_path)

        # Syncing works differently depending on whether
        # We are using a single file or a directory of files
        if blog_file:
            responses = [self.push_blog_file(blog_path, posts)]

        else:
            responses = self.push_post_files(blog_dir, posts)

        return responses

    def get_posts(self, path):
        """
        Fetch posts from single file or directory
        """
        folder, filename = os.path.split(path)

        if filename:
            content = self.get_file(path)
            posts = self.parser.unpack(content)
        else:
            posts = []
            file_list = self._list_path(path)
            for filename in file_list:
                content = self.get_file(path)
                post = self.parser.parse(content)
                posts.append(post)
        return posts

    def pull(self, blog):
        """
        Update local posts from remote source

        blog: Blog instance
        """
        remote_posts = self.get_posts(blog.backend_file)

        updated_list = []
        for remote_post in remote_posts:
            updated = self._update_from_dict(blog, remote_post)
            updated_list.append(updated)

        return updated_list



import datetime

from django.contrib.auth.models import User
from django.core import serializers
from django.db import models
from django.db.models.query import QuerySet
from django.utils import timezone

from social.apps.django_app.default.models import UserSocialAuth
from markupfield.fields import MarkupField

from django.conf import settings

from stardate.utils import get_post_model


class Blog(models.Model):
    authors = models.ManyToManyField(User, blank=True, null=True)
    # Dot notation path to backend Class
    backend_class = models.CharField(
        max_length=255,
        blank=True,
        default=settings.STARDATE_BACKEND
    )
    # Path to file or directory used by backend to determine
    # how and where to store / retrieve posts
    backend_file = models.CharField(blank=True, max_length=255)
    name = models.CharField(max_length=255)
    user = models.ForeignKey(User, related_name="+")
    slug = models.SlugField(unique=True)
    social_auth = models.ForeignKey(UserSocialAuth, blank=True, null=True)

    def __init__(self, *args, **kwargs):
        super(Blog, self).__init__(*args, **kwargs)
        # Instantiate the backend
        from stardate.backends import get_backend
        self.backend = get_backend(self.backend_class)
        # If backend uses a social auth to connect,
        # initialize it here
        try:
            self.backend.set_social_auth(self.social_auth)
        except AttributeError:
            pass

    def __unicode__(self):
        return self.name

    @models.permalink
    def get_absolute_url(self):
        return ('post-archive-index', (), {'blog_slug': self.slug})

    def get_backend_choice(self):
        choices = self.backend.get_source_list()
        return choices[int(self.backend_file)][1]

    def get_serialized_posts(self):
        """
        Returns a list of dictionaries representing post objects on the blog.
        """
        return serializers.serialize("python", self.get_posts().filter(
            deleted=False), fields=('title', 'publish', 'stardate', 'body'))

    def get_posts(self):
        """
        returns a queryset of related posts based on the installed
        ``Post`` model
        """
        Post = get_post_model()
        qs = QuerySet(model=Post).filter(blog=self)
        return qs

    def save_post_objects(self, post_list):
        Post = get_post_model()

        for post in post_list:
            post['blog_id'] = self.id
            try:
                p = Post.objects.get(stardate=post.get('stardate'))
            except Post.DoesNotExist:
                p = Post(**post)
            p.__dict__.update(**post)
            p.save()
            print 'Saved: %s' % p.title

    def sync_backend(self):
        post_list = self.get_serialized_posts()
        path = self.get_backend_choice()
        self.backend.sync(path, post_list)


class PostManager(models.Manager):

    def published(self):
        return self.get_query_set().filter(
            deleted=False,
            publish__lte=timezone.now()).order_by('-publish')


class BasePost(models.Model):
    authors = models.ManyToManyField(User, blank=True, null=True, related_name="%(app_label)s_%(class)s_related")
    blog = models.ForeignKey(Blog, related_name="%(app_label)s_%(class)s_related")
    body = MarkupField(default_markup_type='markdown')
    created = models.DateTimeField(auto_now=True)
    deleted = models.BooleanField(default=False)
    objects = PostManager()
    publish = models.DateTimeField(blank=True, null=True)
    slug = models.SlugField(unique=True)
    stardate = models.CharField(max_length=255)
    title = models.CharField(max_length=255)

    class Meta:
        abstract = True
        app_label = 'stardate'
        ordering = ['-publish']

    def __init__(self, *args, **kwargs):
        super(BasePost, self).__init__(*args, **kwargs)
        # Post must use the same backend as the Blog
        if hasattr(self, 'blog'):
            self.backend = self.blog.backend

    def __unicode__(self):
        return self.title

    def clean(self, *args, **kwargs):
        import uuid
        from django.template.defaultfilters import slugify

        if not self.stardate:
            self.stardate = str(uuid.uuid1())
        if not self.slug:
            self.slug = slugify(self.title)
        if not self.body.raw.endswith('\n'):
            self.body.raw += '\n'

    def mark_deleted(self):
        self.deleted = True
        return self

    def save(self, push=True, *args, **kwargs):
        if not hasattr(self, 'backend'):
            self.backend = self.blog.backend

        if not self.created:
            self.created = datetime.datetime.now()

        # Validate first so things don't break on push
        self.clean()
        self.clean_fields()
        self.validate_unique()

        if push:
            # Initialize our backend with user's social auth
            self.backend.set_social_auth(self.blog.social_auth)
            # Sync this post with our backend
            self.backend.push([self])
        super(BasePost, self).save(*args, **kwargs)

    @models.permalink
    def get_absolute_url(self):
        self.publish = self.publish.astimezone(timezone.get_current_timezone())
        return ('post-detail', (), {
            'blog_slug': self.blog.slug,
            'year': self.publish.year,
            'day': self.publish.day,
            'month': self.publish.strftime('%b').lower(),
            'post_slug': self.slug})

    def get_next_post(self):
        next = self.blog.get_posts().filter(publish__gt=self.publish).exclude(
            id__exact=self.id).order_by('publish')
        if next:
            return next[0]
        return False

    def get_prev_post(self):
        prev = self.blog.get_posts().filter(publish__lt=self.publish).exclude(
                id__exact=self.id).order_by('-publish')
        if prev:
            return prev[0]
        return False



import datetime
import os
import tempfile

from django.contrib.auth.models import User
from django.test import TestCase
from django.utils import timezone

from social.apps.django_app.default.models import UserSocialAuth

from stardate.models import Blog
from stardate.parsers import FileParser
from stardate.tests.factories import create_blog, create_post, create_user, \
    create_user_social_auth
from stardate.tests.mock_backends import MockDropboxClient, \
    MockDropboxBackend, MockLocalFileBackend
from stardate.utils import get_post_model

Post = get_post_model()


class DropboxBackendTestCase(TestCase):
    def setUp(self):
        backend_file, backend_file_path = tempfile.mkstemp(suffix='.txt')
        self.file_path = backend_file_path
        self.backend = MockDropboxBackend()
        social_auth = create_user_social_auth(user=create_user())
        self.backend.set_social_auth(social_auth)

        self.blog = create_blog(
            backend_class="stardate.tests.mock_backends.MockDropboxBackend",
            backend_file=backend_file_path
        )
        create_post(blog=self.blog)

    def tearDown(self):
        Blog.objects.all().delete()
        UserSocialAuth.objects.all().delete()
        User.objects.all().delete()

    def test_get_access_token(self):
        access_token = self.backend.get_access_token()

        self.assertEqual(access_token['oauth_token_secret'], 'oauth_token_secret_string')
        self.assertEqual(access_token['oauth_token'], 'oauth_token_string')

    def test_cursor(self):
        self.assertEqual(self.backend.get_cursor(), None)
        self.backend.save_cursor('testing_cursor')
        self.assertEqual(self.backend.get_cursor(), 'testing_cursor')

    def test_get_dropbox_client(self):
        self.assertIsInstance(self.backend.get_dropbox_client(), MockDropboxClient)

    def test_get_name(self):
        self.assertEqual(self.backend.get_name(), 'dropbox')

    def test_delta(self):
        self.assertEqual(self.backend.get_cursor(), None)
        delta = self.backend.delta()
        self.assertEqual(self.backend.get_cursor(), 'VAU6GZG5NK31AW2YD8H7UDWE0W74VV')
        self.assertTrue(delta.get('entries'))
        self.assertEqual(delta.get('entries')[0][0], '/test_file.md')

    def test_get_file(self):
        backend_file = self.backend.get_file(self.blog.backend_file)
        packed = self.backend.parser.pack(self.backend.get_posts(self.file_path))
        self.assertEqual(backend_file, packed)

    def test_get_file_changed(self):
        backend_file_path = self.blog.backend_file
        self.backend.client.put_file(backend_file_path, 'new string')
        self.assertEqual(self.backend.get_file(backend_file_path), 'new string')

    def test_get_posts(self):
        post_list = self.backend.get_posts(self.blog.backend_file)
        self.assertEqual(len(post_list), 1)
        self.assertEqual(post_list[0]['title'], 'Test post title')

    def test_get_source_list(self):
        source_list = self.backend.get_source_list()
        self.assertEqual(len(source_list), 3)

    def test_pull(self):
        blog = self.blog
        pulled_posts = blog.backend.pull(blog)
        self.assertEqual(len(pulled_posts), 1)
        self.assertEqual(pulled_posts[0].title, 'Test post title')

    def test_push(self):
        create_post(blog=self.blog, title="Test one")
        create_post(blog=self.blog, title="Test two")
        self.blog.backend.push(self.blog.get_posts().all())

        backend_file = self.backend.get_file(self.blog.backend_file)
        packed_string = self.backend.parser.pack(
            self.backend.serialize_posts(self.blog.get_posts().all()))
        self.assertEqual(backend_file, packed_string)

    def test_pull_then_push(self):
        test_string = 'title: My test post\npublish: June 1, 2013\n\n\nPost body.\n'
        self.backend.client.put_file(self.blog.backend_file, test_string)
        self.assertEqual(
            self.backend.client.get_file(self.blog.backend_file).read(),
            test_string)
        for post in self.blog.get_posts().all():
            post.delete()
        self.assertFalse(self.blog.get_posts().all())
        self.backend.pull(self.blog)
        self.assertEqual(len(self.blog.get_posts().all()), 1)
        self.backend.push(self.blog.get_posts().all())
        # After push, file should have stardate metadata
        packed_string = self.backend.parser.pack(
            self.backend.serialize_posts(self.blog.get_posts().all()))
        self.assertEqual(
            self.backend.client.get_file(self.blog.backend_file).read(),
            packed_string)

    def test_pull_then_pull(self):
        test_string = 'title: Title\n\n\nBody.\n'
        self.backend.client.put_file(self.blog.backend_file, test_string)
        for post in self.blog.get_posts().all():
            post.delete()
        self.backend.pull(self.blog)
        self.backend.pull(self.blog)

    def test_push_then_pull(self):
        pass

    def test_push_blog_file(self):
        posts = self.blog.get_posts().all()
        self.backend.push_blog_file(self.blog.backend_file, posts)
        f = open(self.blog.backend_file, 'r')
        packed_string = self.backend.parser.pack(
            self.backend.serialize_posts(self.blog.get_posts().all()))
        self.assertEqual(f.read(), packed_string)

    # def test_push_blog_files(self):
    #     self.assertEqual(1, 2)


    def test_save_cursor(self):
        self.backend.save_cursor('test_cursor')
        self.assertEqual(self.backend.cursor, 'test_cursor')

    def test_set_social_auth(self):
        user = User.objects.get(username='bturner')
        social_auth = UserSocialAuth.objects.get(user__exact=user.id)

        self.backend.social_auth = None
        self.assertEqual(self.backend.social_auth, None)

        self.backend.set_social_auth(social_auth)
        self.assertIsInstance(self.backend.social_auth, UserSocialAuth)

    def test_get_post_path(self):
        post_list = self.blog.get_posts().all()

        post_path = self.backend.get_post_path('posts', post_list[0])
        self.assertEqual(post_path, u'posts/test-post-title.md')

        # Try with no folder
        post_path = self.backend.get_post_path('', post_list[0])
        self.assertEqual(post_path, u'test-post-title.md')

    def test_serialize_posts(self):
        serialized_posts = self.backend.serialize_posts(self.blog.get_posts().all())
        self.assertIn('title', serialized_posts[0])
        self.assertIn('stardate', serialized_posts[0])
        self.assertIn('publish', serialized_posts[0])
        self.assertIn('body', serialized_posts[0])

    # def test_sync(self):
    #     user = User.objects.get(username='bturner')
    #     blog = create_blog(owner=user)
    #     blog.backend.client_class = MockDropboxClient
    #     create_post(blog=blog)
    #     post_list = blog.post_set.all()

    #     sync = self.backend.sync(post_list)

    #     self.assertEqual(sync.read(), 'stardate: 352b967d-87bf-11e2-81f3-b88d120c8298\npublish: 2013-01-04 12:00 AM\ntitle: Fourth post\n\n\nDo it.\n')


class LocalFileBackendTestCase(TestCase):
    def setUp(self):
        fd, file_path = tempfile.mkstemp(suffix='.md')
        self.blog = create_blog(
            backend_class="stardate.tests.mock_backends.MockLocalFileBackend",
            backend_file=file_path)
        create_post(title="Hello world", blog=self.blog)

        self.post_list = self.blog.get_posts().all()

    def tearDown(self):
        Blog.objects.all().delete()
        Post.objects.all().delete()
        User.objects.all().delete()
        UserSocialAuth.objects.all().delete()

    def test_is_mock_local_file_backend(self):
        self.assertIsInstance(self.blog.backend, MockLocalFileBackend)

    def test_get_name(self):
        self.assertEqual(self.blog.backend.get_name(), 'localfile')

    def test_get_post_path(self):
        serialized_posts = self.blog.backend.serialize_posts(
            self.post_list)
        path = self.blog.backend._get_post_path('', serialized_posts[0])
        self.assertEqual(path, 'hello-world.md')
        path = self.blog.backend._get_post_path('posts', serialized_posts[0])
        self.assertEqual(path, 'posts/hello-world.md')

    def test_posts_from_file(self):
        temp_file = tempfile.mkstemp(suffix='.md')[1]
        f = open(temp_file, 'w')
        f.write('title: Test post\n\n\nThe body content.')
        f.close()

        posts = self.blog.backend.get_posts(temp_file)
        self.assertEqual(posts, [{'title': 'Test post', 'body': 'The body content.'}])

    def test_posts_from_dir(self):
        temp_dir = tempfile.mkdtemp()
        fd, file_path = tempfile.mkstemp(dir=temp_dir, suffix='.md')
        f = open(file_path, 'w')
        f.write('title: Test post\n\n\nThe body content.')
        f.close()

        fd, file_path = tempfile.mkstemp(dir=temp_dir, suffix='.md')
        f = open(file_path, 'w')
        f.write('title: Another test post\n\n\nA different body.')
        f.close()

        posts = self.blog.backend.get_posts(temp_dir)
        self.assertEqual(len(posts), 2)
        self.assertTrue('title' in posts[0])
        self.assertTrue('title' in posts[1])
        self.assertTrue('body' in posts[0])
        self.assertTrue('body' in posts[1])

        # cleanup temp files
        for f in os.listdir(temp_dir):
            f = os.path.join(temp_dir, f)
            os.remove(f)
        os.removedirs(temp_dir)

    def test_pull(self):
        temp_dir = tempfile.mkdtemp()
        fd, file_path = tempfile.mkstemp(dir=temp_dir, suffix='.md')
        blog = create_blog(name='Pull',
                           backend_class='stardate.tests.mock_backends.MockLocalFileBackend',
                           backend_file=file_path,
                           slug='pull')

        f = open(file_path, 'w')
        f.write('title: Post title\npublish: 2013-01-01 6:00 AM\n\n\nA post for pulling in.')
        f.close()

        pulled_posts = blog.backend.pull(blog)

        self.assertIsNotNone(pulled_posts[0].stardate)
        self.assertEqual(pulled_posts[0].title, 'Post title')
        self.assertEqual(pulled_posts[0].body.raw, 'A post for pulling in.\n')
        self.assertEqual(pulled_posts[0].publish, datetime.datetime(2013, 1, 1, 14, 0, tzinfo=timezone.utc))

    def test_push(self):
        fd, file_path = tempfile.mkstemp(suffix='.md')
        blog = create_blog(name='Push',
                           backend_class='stardate.tests.mock_backends.MockLocalFileBackend',
                           backend_file=file_path,
                           slug='push')

        # creating a post calls `push`
        create_post(title='A test push post', blog=blog, body='Testing a push post.')

        f = open(file_path, 'r')
        content = f.read()
        parser = FileParser()
        parsed = parser.unpack(content)
        self.assertEqual(parsed[0]['title'], u'A test push post')
        self.assertEqual(parsed[0]['body'], u'Testing a push post.\n')
        self.assertTrue('stardate' in parsed[0])

    def test_serialize_posts(self):
        serialized_posts = self.blog.backend.serialize_posts(self.post_list)
        self.assertIn('title', serialized_posts[0])
        self.assertIn('created', serialized_posts[0])
        self.assertIn('stardate', serialized_posts[0])
        self.assertIn('publish', serialized_posts[0])
        self.assertIn('body', serialized_posts[0])



from django import forms

from markupfield.widgets import AdminMarkupTextareaWidget

from stardate.backends import get_backend
from stardate.models import Blog
from stardate.utils import get_post_model


backend = get_backend()
Post = get_post_model()


class BlogForm(forms.ModelForm):
    """
    A form that populates the backend file choices select field.
    """
    class Meta:
        model = Blog

    # backend_file = forms.ChoiceField(required=False)

    def __init__(self, *args, **kwargs):
        super(BlogForm, self).__init__(*args, **kwargs)
        try:
            qs = Blog.objects.get(pk=self.instance.pk)
            backend.set_social_auth(qs.social_auth)
        except:
            pass

        # self.fields['backend_file'].choices = backend.get_source_list()


class PostForm(forms.ModelForm):
    body = forms.CharField(widget=AdminMarkupTextareaWidget)

    class Meta:
        model = Post



from django.core.exceptions import ImproperlyConfigured


def get_post_model():
    """
    Return the Post model that is active in this project
    """
    from django.conf import settings
    from django.db.models import get_model

    try:
        POST_MODEL = getattr(settings, 'STARDATE_POST_MODEL')
    except AttributeError:
        raise NotImplementedError('STARDATE_POST_MODEL is not defined.')

    try:
        app_label, model_name = POST_MODEL.split('.')
    except ValueError:
        raise ImproperlyConfigured("STARDATE_POST_MODEL must be of the form 'app_label.model_name'")

    post_model = get_model(app_label, model_name)
    if post_model is None:
        raise ImproperlyConfigured("STARDATE_POST_MODEL refers to model '%s' that has not been installed" % POST_MODEL)
    return post_model



from django.conf.urls import include, patterns, url
from django.views import generic

from stardate.models import Blog


urlpatterns = patterns('',
    url(r'^(?P<blog_slug>[-\w]+)/', include('stardate.urls.blog_urls')),
    url(r'^$', generic.ListView.as_view(model=Blog), name='blog-list'),
)



import logging

from django.core.management.base import BaseCommand
from django.contrib.auth.models import User
from optparse import make_option

from stardate.models import Blog

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    """
    Usage:
    ./manage.py import_posts --user [username] --user [otheruser]

    When importing posts from the backend, a backend should check that
    there is new data on the backend itself, if there is new data, then
    the backend may fetch new data to be updated.

    """
    option_list = BaseCommand.option_list + (
        make_option(
            '--user',
            action='append',
            dest='user',
            default=[],
            help='Specify a username to fetch blog posts'
        ),
    )

    def handle(self, *args, **options):
        if options['user']:
            users = User.objects.filter(username__in=options['user'])
        else:
            users = User.objects.all()

        for user in users:
            for blog in Blog.objects.filter(user=user):
                logger.info(u'Updating posts for {0}'.format(blog))
                blog.backend.pull(blog)



from django.conf import settings
from django.utils.importlib import import_module
from django.core.serializers import serialize
import os
import logging

from stardate.utils import get_post_model


Post = get_post_model()

logger = logging.getLogger('stardate')

STARDATE_BACKEND = getattr(settings, 'STARDATE_BACKEND', 'stardate.backends.dropbox.DropboxBackend')


def get_backend(backend=STARDATE_BACKEND):
    i = backend.rfind('.')
    module, attr = backend[:i], backend[i + 1:]
    try:
        mod = import_module(module)
    except ImportError, e:
        print e
    try:
        backend_class = getattr(mod, attr)
    except AttributeError:
        print module
    return backend_class()


class StardateBackend(object):
    def __init__(self, name=None, parser=None, social_auth=None):
        self.name = name
        self.parser = parser
        self.social_auth = social_auth

    def set_social_auth(self, *args, **kwargs):
        raise NotImplementedError

    def get_name(self):
        return self.name

    def serialize_posts(self, posts):
        """
        Returns dictionary version of Post objects
        """
        posts_as_dicts = []
        serialized = serialize(
            'python',
            posts,
            fields=('title', 'created', 'slug', 'publish', 'stardate', 'body')
        )
        for post in serialized:
            posts_as_dicts.append(post['fields'])
        return posts_as_dicts

    def _update_from_dict(self, blog, post_dict, post=None):
        """
        Create or update a Post from a dictionary
        """
        created = False
        # If a post is not provided, try an fetch it
        if not post:
            if 'stardate' in post_dict:
                post = Post.objects.filter(
                    blog=blog,
                    stardate=post_dict['stardate']
                )
                if post:
                    post = post[0]
            if not post:
                post_dict['blog'] = blog
                post, created = Post.objects.get_or_create(**post_dict)

        # Update from dict values
        if not created:
            for att, value in post_dict.items():
                setattr(post, att, value)
            post.save(push=False)
        logger.info('Blog: %s, Post: %s, created=%s', post.blog, post, created)
        return post

    def push_blog_file(self, file_path, posts):
        """
        Update posts in a single blog file
        """
        remote_posts = self.get_posts(file_path)

        # Use serialized version of posts to find
        # and update
        local_posts = self.serialize_posts(posts)

        # Update remote_posts with local versions
        ## FIXME: n^2 crawl, use stardate as keys
        ## in dicts instead of lists?
        for local_post in local_posts:
            exists = False
            # First we try to see if match by stardate exists
            for remote_post in remote_posts:
                if 'stardate' in remote_post:
                    if local_post['stardate'] == remote_post['stardate']:
                        exists = True
                        remote_post.update(local_post)
                        break

            # If post was created remotely and was pulled in
            # then it has a stardate, but remote post does not.
            # Try to match up using title
            if not exists:
                for remote_post in remote_posts:
                    if 'title' in remote_post:
                        if local_post['title'] == remote_post['title']:
                            exists = True
                            remote_post.update(local_post)
                            break

            # Add new remote post if it does not exist yet
            if not exists:
                remote_posts.append(local_post)

        # Turn post list back into string
        content = self.parser.pack(remote_posts)
        self.write_file(file_path, content)
        return


    def push_post_files(self, folder, posts):
        """
        Update posts in multiple files
        """
        local_posts = self.serialized_posts(posts)

        for local_post in local_posts:
            # Generate the post file path dynamically
            post_path = self._get_post_path(folder, local_post)

            # Get the existing remote post as a post dict
            remote_post = self.get_post(post_path)

            # Update the contents of the remote post
            remote_post.update(local_post)
            content = self.parser.render(remote_post)
            self.write_file(content)
        return


    def push(self, posts):
        """
        Render posts to files

        posts: List of Post object instances
        """
        # Grab the file or folder path associated
        # with a blog
        blog_path = posts[0].blog.backend_file

        # Separate blog path into directory and filename
        blog_dir, blog_file = os.path.split(blog_path)

        # pushing works differently depending on whether
        # We are using a single file or a directory of files
        if blog_file:
            responses = [self.push_blog_file(blog_path, posts)]

        else:
            responses = self.push_post_files(blog_dir, posts)

        return responses

    def pull(self, blog):
        """
        Update local posts from remote source

        blog: Blog instance
        """
        remote_posts = self.get_posts(blog.backend_file)

        updated_list = []
        for remote_post in remote_posts:
            updated = self._update_from_dict(blog, remote_post)
            updated_list.append(updated)

        return updated_list


class BaseStardateParser(object):
    def pack(self):
        raise NotImplementedError

    def unpack(self):
        raise NotImplementedError



from django.contrib.syndication.views import Feed
from django.shortcuts import get_object_or_404
from markdown import markdown

from stardate.models import Blog
from stardate.utils import get_post_model

Post = get_post_model()


class LatestPostsFeed(Feed):
    def get_object(self, request, blog_slug):
        return get_object_or_404(Blog, slug=blog_slug)

    def title(self, obj):
        return "%s: Recent posts" % obj.name

    def link(self, obj):
        return obj.get_absolute_url()

    def items(self, obj):
        return Post.objects.published().filter(blog=obj)[:5]

    def item_title(self, item):
        return item.title

    def item_link(self, item):
        return item.get_absolute_url()

    def item_pubdate(self, item):
        return item.publish

    def item_description(self, item):
        return markdown(item.body)[:300]



from setuptools import setup

setup(
    name='django-stardate',
    version='0.1.0.a2',
    author=u'Benjamin Turner',
    author_email='benturn@gmail.com',
    packages=[
        'stardate',
        'stardate.backends',
        'stardate.management',
        'stardate.management.commands',
        'stardate.tests',
        'stardate.urls',
    ],
    package_data={
        'stardate': [
            'templates/stardate/includes/*.html',
            'templates/stardate/*.html',
        ],
    },
    url='https://github.com/blturner/django-stardate',
    download_url='https://github.com/blturner/django-stardate/archive/v0.1.0.a2.tar.gz',
    license='BSD',
    description='Another django blog app.',
    # long_description=open('README').read(),
    zip_safe=False,
    install_requires=[
        'Django>=1.4,<1.7',
        'django-markupfield==1.1.1',
        'dropbox>=1.4,<2.0',
        'Markdown==2.0.3',
        'PyYAML==3.11',
        'python-dateutil==2.1',
        'python-social-auth>=0.1,<0.2',
        'pytz<2014.4',
    ],
    test_suite='stardate.tests'
)



import datetime
import uuid
import yaml

from dateutil.parser import parse
from django.utils.timezone import get_current_timezone, make_aware, utc

from stardate.backends import BaseStardateParser

DELIMITER = "\n---\n"
TIMEFORMAT = '%Y-%m-%d %I:%M %p'  # 2012-01-01 09:00 AM
current_timezone = get_current_timezone()


class FileParser(BaseStardateParser):
    def __init__(self):
        self.delimiter = DELIMITER
        self.timeformat = TIMEFORMAT

    def render(self, post):
        """
        Turn a post dictionary into a rendered string
        """
        post = post.copy()

        # FIXME?: this belongs in serialization process
        try:
            post['publish'] = post['publish'].astimezone(current_timezone)
            post['publish'] = datetime.datetime.strftime(post['publish'], self.timeformat)
        except:
            pass

        # Body gets processed separately
        body = post.pop('body')

        # Generate meta data lines
        # One key/value pair per line
        meta = []
        for key in sorted(post.keys()):
            try:
                value = post[key]
                if value:
                    field_string = '{0}: {1}'.format(key, value)
                    meta.append(field_string)
            except KeyError:
                pass
        meta = '\n'.join(meta)

        # Body is separated from meta by three lines
        rendered = '{0}\n\n\n{1}'.format(meta, body)
        return rendered

    def pack(self, posts):
        """
        Render a list of post dictionaries to a single string
        """
        post_list = []

        for post in posts:
            post_list.append(self.render(post))

        document = self.delimiter.join(post_list)
        return document

    def parse(self, string):
        """
        Parse a single string into a dictionary representing a post object.
        """
        try:
            # split each string from it's meta data
            bits = string.split('\n\n\n')

            # load meta data into post dictionary
            post_data = yaml.load(bits[0])

            # post body is everything else
            # Join incase other parts of post are separated
            # by three return characters \n\n\n
            post_data['body'] = ''.join(bits[1:])
        except:
            post_data = {}

        # FIXME: this belongs in deserialization, perhaps
        # on model?
        if 'publish' in post_data:
            if not isinstance(post_data['publish'], datetime.datetime):
                post_data['publish'] = parse(post_data['publish'])
                post_data['publish'] = make_aware(post_data['publish'], current_timezone).astimezone(utc)
        return post_data

    def unpack(self, string):
        """
        Returns a list of parsed post dictionaries.
        """
        self.parsed_list = []

        for post in string.split(self.delimiter):
            if self.parse(post):
                self.parsed_list.append(self.parse(post))
        return self.parsed_list



import tempfile

from django.contrib.auth.models import User
from django.template.defaultfilters import slugify

from social.apps.django_app.default.models import UserSocialAuth

from stardate.models import Blog
from stardate.utils import get_post_model

Post = get_post_model()


def create_user(**kwargs):
    defaults = {
        "username": "bturner",
    }
    defaults.update(kwargs)
    user, created = User.objects.get_or_create(**defaults)
    return user


def create_user_social_auth(**kwargs):
    defaults = {
        "provider": "dropbox",
        "uid": "1234",
        "user": kwargs['user'],
        "extra_data": {
            "access_token": {
                u'oauth_token_secret': u'oauth_token_secret_string',
                u'oauth_token': u'oauth_token_string',
                u'uid': u'123'
            }
        }
    }
    defaults.update(kwargs)
    social_auth, created = UserSocialAuth.objects.get_or_create(**defaults)
    return social_auth


def create_blog(**kwargs):
    backend_file, backend_file_path = tempfile.mkstemp(suffix='.txt')
    defaults = {
        "name": "Test blog",
        "backend_file": backend_file_path,
    }
    defaults['slug'] = slugify(defaults['name'])
    defaults.update(kwargs)

    if "user" not in defaults:
        defaults["user"] = create_user()

    defaults["social_auth"] = create_user_social_auth(
        user=defaults["user"])

    blog, created = Blog.objects.get_or_create(**defaults)
    return blog


def create_post(**kwargs):
    push = kwargs.pop('push', True)
    defaults = {
        "blog": kwargs['blog'],
        "body": "Test post body.",
        "title": "Test post title",
    }
    defaults.update(kwargs)
    defaults["slug"] = slugify(defaults["title"])
    post = Post(**defaults)
    post.save(push=push)
    return post
